package com.iusmob.ad_klein_flutter_sdk;

import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.iusmob.adklein.ad.AdKleinBannerAd;
import com.iusmob.adklein.ad.AdKleinBannerAdListener;
import com.iusmob.adklein.ad.AdKleinError;
import com.iusmob.ad_klein_flutter_sdk.DisplayUtil;

import io.flutter.plugin.platform.PlatformView;

public class FlutterBannerAd extends FlutterAd implements PlatformView, FlutterDestroyableAd {

    @NonNull
    private final AdInstanceManager manager;
    /**
     * 广告位id
     */
    @NonNull
    private final String adPosId;
    /**
     * 场景id可选
     */
    @NonNull
    private final String sceneId;
    /**
     * 广告宽
     */
    @NonNull
    private final Double adWidth;
    /**
     * 广告高
     */
    @NonNull
    private final Double adHeight;
    /**
     * 广告容器
     */
    private FrameLayout flContainer;
    /**
     * 广告对象
     */
    private AdKleinBannerAd adKleinBannerAd;

    static class Builder {
        @Nullable
        private AdInstanceManager manager;
        @Nullable
        private String adPosId;
        @Nullable
        private String sceneId;
        @Nullable
        private Double adWidth;
        @Nullable
        private Double adHeight;

        public Builder setManager(@NonNull AdInstanceManager manager) {
            this.manager = manager;
            return this;
        }

        public Builder setAdPosId(@NonNull String adPosId) {
            this.adPosId = adPosId;
            return this;
        }

        public Builder setAdWidth(@NonNull Double adWidth) {
            this.adWidth = adWidth;
            return this;
        }

        public Builder setAdHeight(@NonNull Double adHeight) {
            this.adHeight = adHeight;
            return this;
        }

        public Builder setSceneId(@NonNull String sceneId) {
            this.sceneId = sceneId;
            return this;
        }

        FlutterBannerAd build() {
            if (manager == null) {
                throw new IllegalStateException("AdInstanceManager cannot not be null.");
            } else if (adPosId == null) {
                throw new IllegalStateException("adPosId cannot not be null.");
            }

            final FlutterBannerAd bannerAd = new FlutterBannerAd(manager, adPosId, sceneId, adWidth, adHeight);
            return bannerAd;
        }
    }

    private FlutterBannerAd(
            @NonNull AdInstanceManager manager, @NonNull String adPosId, @NonNull String sceneId, @NonNull Double adWidth, @NonNull Double adHeight) {
        this.manager = manager;
        this.adPosId = adPosId;
        this.sceneId = sceneId;
        this.adWidth = adWidth;
        this.adHeight = adHeight;

        flContainer = new FrameLayout(manager.activity);
        flContainer.setLayoutParams(new ViewGroup.LayoutParams(
                        DisplayUtil.dp2px(manager.activity, new Double(this.adWidth).intValue()),
                        DisplayUtil.dp2px(manager.activity, new Double(this.adHeight).intValue())
                )
        );

    }

    @Override
    void load() {
        adKleinBannerAd = new AdKleinBannerAd(
                manager.activity, adPosId, flContainer,
                new AdKleinBannerAdListener() {
                    @Override
                    public void onAdClose() {
                        manager.onAdClose(FlutterBannerAd.this);
                    }

                    @Override
                    public void onError(AdKleinError adKleinError) {
                        manager.onAdFailed(FlutterBannerAd.this, adKleinError);
                    }

                    @Override
                    public void onAdLoaded() {
                        manager.onAdReceive(FlutterBannerAd.this);
                        adKleinBannerAd.show();
                    }

                    @Override
                    public void onAdShow() {
                        manager.onAdExpose(FlutterBannerAd.this);
                    }

                    @Override
                    public void onAdClicked() {
                        manager.onAdClick(FlutterBannerAd.this);
                    }
                },
                DisplayUtil.dp2px(manager.activity, new Double(adWidth).intValue()), 0.0F);
        adKleinBannerAd.setAutoRefreshInterval(30);
        adKleinBannerAd.load();

    }

    @Override
    public void release() {
        if (adKleinBannerAd != null) {
            adKleinBannerAd = null;
        }
    }

    @Override
    public View getView() {
        return flContainer;
    }

    @Override
    public void dispose() {

    }
}
