package com.iusmob.ad_klein_flutter_sdk;

import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.iusmob.adklein.ad.AdKleinError;
import com.iusmob.adklein.ad.AdKleinInterstitialAd;
import com.iusmob.adklein.ad.AdKleinInterstitialAdListener;
import com.iusmob.ad_klein_flutter_sdk.DisplayUtil;

public class FlutterInterstitialAd extends FlutterAd.FlutterOverlayAd implements FlutterDestroyableAd {

    @NonNull
    private final AdInstanceManager manager;
    /**
     * 广告位id
     */
    @NonNull
    private final String adPosId;
    /**
     * 广告对象
     */
    private AdKleinInterstitialAd kleinInterstitial;

    @Override
    void show() {
        if (kleinInterstitial != null) {
            // TODO 插屏广告对象一次成功拉取的广告数据只允许展示一次
            kleinInterstitial.show();
        }
    }

    static class Builder {
        @Nullable
        private AdInstanceManager manager;
        @Nullable
        private String adPosId;

        public Builder setManager(@NonNull AdInstanceManager manager) {
            this.manager = manager;
            return this;
        }

        public Builder setAdPosId(@NonNull String adPosId) {
            this.adPosId = adPosId;
            return this;
        }

        FlutterInterstitialAd build() {
            if (manager == null) {
                throw new IllegalStateException("AdInstanceManager cannot not be null.");
            } else if (adPosId == null) {
                throw new IllegalStateException("adPosId cannot not be null.");
            }

            final FlutterInterstitialAd interstitialAd = new FlutterInterstitialAd(manager, adPosId);
            return interstitialAd;
        }
    }

    private FlutterInterstitialAd(
            @NonNull AdInstanceManager manager, @NonNull String adPosId) {
        this.manager = manager;
        this.adPosId = adPosId;
    }

    @Override
    void load() {
        // 设置插屏广告监听
        kleinInterstitial = new AdKleinInterstitialAd(
                manager.activity, adPosId,
                new AdKleinInterstitialAdListener() {
                    @Override
                    public void onAdClosed() {
                        manager.onAdClose(FlutterInterstitialAd.this);
                    }

                    @Override
                    public void onRenderFail(AdKleinError adKleinError) {
                    }

                    @Override
                    public void onRenderSuccess() {
                    }

                    @Override
                    public void onError(AdKleinError adKleinError) {
                        manager.onAdFailed(FlutterInterstitialAd.this, adKleinError);
                    }

                    @Override
                    public void onAdLoaded() {
                        manager.onAdReceive(FlutterInterstitialAd.this);
                    }

                    @Override
                    public void onAdShow() {
                        manager.onAdExpose(FlutterInterstitialAd.this);
                    }

                    @Override
                    public void onAdClicked() {
                        manager.onAdClick(FlutterInterstitialAd.this);
                    }
                }, (int) (DisplayUtil.getWindowsWidth(manager.activity) * 3 / 4), DisplayUtil.getWindowsWidth(manager.activity)
        );
        // 加载插屏广告，参数为广告位ID
        kleinInterstitial.load();
    }

    @Override
    public void release() {
    }
}
