package com.iusmob.ad_klein_flutter_sdk;

import android.app.Activity;
import android.content.Context;
import android.graphics.Point;
import android.util.Log;
import android.view.View;
import android.widget.FrameLayout;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.iusmob.adklein.ad.AdKleinError;
import com.iusmob.adklein.ad.AdKleinNativeExpressAd;
import com.iusmob.adklein.ad.AdKleinNativeExpressAdData;
import com.iusmob.adklein.ad.AdKleinNativeExpressAdListener;

import java.util.List;

import io.flutter.plugin.platform.PlatformView;

public class FlutterNativeAd extends FlutterAd implements PlatformView, FlutterDestroyableAd {

    @NonNull
    private final AdInstanceManager manager;
    /**
     * 广告位id
     */
    @NonNull
    private final String adPosId;
    /**
     * 场景id可选
     */
    @NonNull
    private final String sceneId;
    /**
     * 广告宽
     */
    @NonNull
    private final int adWidth;
    /**
     * 广告高
     */
    @NonNull
    private final int adHeight;
    /**
     * 广告容器
     */
    private FrameLayout flContainer;
    /**
     * 广告对象
     */
    private AdKleinNativeExpressAd expressAd;

    static class Builder {
        @Nullable
        private AdInstanceManager manager;
        @Nullable
        private String adPosId;
        @Nullable
        private String sceneId;

        public Builder setManager(@NonNull AdInstanceManager manager) {
            this.manager = manager;
            return this;
        }

        public Builder setAdPosId(@NonNull String adPosId) {
            this.adPosId = adPosId;
            return this;
        }

        public Builder setSceneId(@NonNull String sceneId) {
            this.sceneId = sceneId;
            return this;
        }

        FlutterNativeAd build() {
            if (manager == null) {
                throw new IllegalStateException("AdInstanceManager cannot not be null.");
            } else if (adPosId == null) {
                throw new IllegalStateException("adPosId cannot not be null.");
            }

            final FlutterNativeAd nativeAd = new FlutterNativeAd(manager, adPosId, sceneId);
            return nativeAd;
        }
    }

    private FlutterNativeAd(
            @NonNull AdInstanceManager manager, @NonNull String adPosId, @NonNull String sceneId) {
        this.manager = manager;
        this.adPosId = adPosId;
        this.sceneId = sceneId;
        this.adWidth = getScreenWidth(manager.activity);
        this.adHeight = 0;

        flContainer = new FrameLayout(manager.activity);
    }

    private int getScreenWidth(Activity activity) {
        Point screenSize = new Point();
        activity.getWindowManager().getDefaultDisplay().getSize(screenSize);
        return (int) screenSize.x;
    }

    @Override
    void load() {
        expressAd = new AdKleinNativeExpressAd(
                manager.activity,
                adPosId,
                new AdKleinNativeExpressAdListener() {
                    @Override
                    public void onAdLoaded(List<AdKleinNativeExpressAdData> adInfoList) {
                        manager.onAdReceive(FlutterNativeAd.this);
                        if (adInfoList != null && adInfoList.size() > 0) {
                            AdKleinNativeExpressAdData expressAdInfo = adInfoList.get(0);
                            showNativeAd(expressAdInfo);
                        }
                    }

                    @Override
                    public void onRenderFail(AdKleinNativeExpressAdData adKleinNativeExpressAdData, AdKleinError adKleinError) {

                    }

                    @Override
                    public void onAdClose(AdKleinNativeExpressAdData adKleinNativeExpressAdData) {
                        manager.onAdClose(FlutterNativeAd.this);
                    }

                    @Override
                    public void onError(AdKleinError adKleinError) {
                        manager.onAdFailed(FlutterNativeAd.this, adKleinError);
                    }

                    @Override
                    public void onAdShow() {
                        manager.onAdExpose(FlutterNativeAd.this);
                    }

                    @Override
                    public void onAdClicked() {
                        manager.onAdClick(FlutterNativeAd.this);
                    }
                }, adWidth, 0.0F
        );
        //设置华为广告为测试环境
        expressAd.setDev(true);
        expressAd.setAdPlayWithMute(true);
        expressAd.load(1);
    }

    private void showNativeAd(AdKleinNativeExpressAdData nativeExpressAdInfo) {
        // 当前是信息流模板广告，getNativeExpressAdView获取的是整个模板广告视图
        final View nativeExpressAdView = nativeExpressAdInfo.getView();
        flContainer.removeAllViews();
        flContainer.addView(nativeExpressAdView);
        // 渲染广告视图, 必须调用, 因为是模板广告, 所以传入ViewGroup和响应点击的控件可能并没有用
        // 务必在最后调用
        nativeExpressAdInfo.render();
        manager.onNativeReceived(FlutterNativeAd.this
                , adWidth
                , px2dip(manager.activity, (adWidth / 10 * 9)));
    }

    public static int px2dip(Context context, float pxValue) {
        final float scale = context.getResources().getDisplayMetrics().density;
        return (int) (pxValue / (scale <= 0 ? 1 : scale) + 0.5f);
    }

    @Override
    public void release() {
        if (expressAd != null) {
            expressAd = null;
        }
        flContainer.removeAllViews();
    }

    @Override
    public View getView() {
        return flContainer;
    }

    @Override
    public void dispose() {

    }

    public int[] unDisplayViewSize(View view) {
        int size[] = new int[2];
        int width = View.MeasureSpec.makeMeasureSpec(0,
                View.MeasureSpec.UNSPECIFIED);
        int height = View.MeasureSpec.makeMeasureSpec(0,
                View.MeasureSpec.UNSPECIFIED);
        view.measure(width, height);
        size[0] = view.getMeasuredWidth();
        size[1] = view.getMeasuredHeight();
        Log.d("unDisplayViewSize", "width:" + width + " height:" + height);
        Log.d("unDisplayViewSize",
                "width:" + view.getMeasuredWidth() + " height:" + view.getMeasuredHeight());
        return size;
    }
}
