package com.iusmob.ad_klein_flutter_sdk;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.iusmob.adklein.ad.AdKleinError;
import com.iusmob.adklein.ad.AdKleinRewardVideoAd;
import com.iusmob.adklein.ad.AdKleinRewardVideoAdListener;

public class FlutterRewardVodAd extends FlutterAd.FlutterOverlayAd implements FlutterDestroyableAd {

    @NonNull
    private final AdInstanceManager manager;
    /**
     * 广告位id
     */
    @NonNull
    private final String adPosId;
    /**
     * 广告对象
     */
    private AdKleinRewardVideoAd rewardVideo;

    @Override
    void show() {
        if (rewardVideo != null) {
            // TODO 激励视频广告对象一次成功拉取的广告数据只允许展示一次
            rewardVideo.show();
        }
    }

    static class Builder {
        @Nullable
        private AdInstanceManager manager;
        @Nullable
        private String adPosId;

        public Builder setManager(@NonNull AdInstanceManager manager) {
            this.manager = manager;
            return this;
        }

        public Builder setAdPosId(@NonNull String adPosId) {
            this.adPosId = adPosId;
            return this;
        }

        FlutterRewardVodAd build() {
            if (manager == null) {
                throw new IllegalStateException("AdInstanceManager cannot not be null.");
            } else if (adPosId == null) {
                throw new IllegalStateException("adPosId cannot not be null.");
            }

            final FlutterRewardVodAd rewardVodAd = new FlutterRewardVodAd(manager, adPosId);
            return rewardVodAd;
        }
    }

    private FlutterRewardVodAd(
            @NonNull AdInstanceManager manager, @NonNull String adPosId) {
        this.manager = manager;
        this.adPosId = adPosId;
    }

    @Override
    void load() {
        rewardVideo = new AdKleinRewardVideoAd(
                manager.activity, adPosId,
                0, true,
                new AdKleinRewardVideoAdListener() {
                    @Override
                    public void onAdClose() {
                        manager.onAdClose(FlutterRewardVodAd.this);
                    }

                    @Override
                    public void onVideoComplete() {

                    }

                    @Override
                    public void onSkippedVideo() {
                    }

                    @Override
                    public void onAdReward() {
                        manager.onAdReward(FlutterRewardVodAd.this);
                    }

                    @Override
                    public void onError(AdKleinError adKleinError) {
                        manager.onAdFailed(FlutterRewardVodAd.this, adKleinError);
                    }

                    @Override
                    public void onAdLoaded() {
                        manager.onAdReceive(FlutterRewardVodAd.this);
                    }

                    @Override
                    public void onAdShow() {
                        manager.onAdExpose(FlutterRewardVodAd.this);
                    }

                    @Override
                    public void onAdClicked() {
                        manager.onAdClick(FlutterRewardVodAd.this);
                    }
                }
        );

        rewardVideo.load();
    }

    @Override
    public void release() {
        if (rewardVideo != null) {
        }
    }
}

