import Flutter
import UIKit
import AdKleinSDK

public class SwiftAdKleinFlutterSdkPlugin: NSObject, FlutterPlugin {
  static var channel: FlutterMethodChannel?

      static let instance = SwiftAdKleinFlutterSdkPlugin()

      public static func register(with registrar: FlutterPluginRegistrar) {
          channel = FlutterMethodChannel(name: "ad_klein_flutter_sdk", binaryMessenger: registrar.messenger())
          registrar.addMethodCallDelegate(instance, channel: channel!)
          registrar.register(ADKleinFlutterAdViewFactory(), withId: "ADKleinAdView")
      }

      public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
          if call.method == "initSdk" {
              let appId = call.arguments
              // 初始化莫比乌斯SDK
              AdKleinSDKConfig.debugMode()
              AdKleinSDKConfig.initMediaId(appId as! String) { error in
                  print("\(String(describing: error))")
              }
              result(nil)
              return
          }

          guard let dic: Dictionary<String, Any> = call.arguments as? Dictionary<String, Any> else {
              result(nil)
              return
          }
          guard let adId = dic["adId"] as? Int else {
              result(nil)
              return
          }
          switch call.method {
          case "loadSplashAd": do {
                  guard let posId = dic["posId"] as? String,
                        let imgName = dic["imgName"] as? String else {
                      result(nil)
                      return
                  }
                  ADKleinFlutterSplashAd.shared.loadAndShow(posid: posId, adId: adId, imgName: imgName)
                  result(nil)
              }
          case "loadBannerAd": do {
                      guard let posId = dic["posId"] as? String,
                            let adWidth = dic["adWidth"] as? Double,
                            let adHeight = dic["adHeight"] as? Double else {
                         result(nil)
                         return
                     }
                      ADKleinFlutterBannerAd.shared.load(posId: posId, adId: adId, adWidth:adWidth, adHeight:adHeight)
                      result(nil)
                  }
          case "removeAd": do {
                  ADKleinFlutterAdMap.shared.deleteAd(adId: adId)
                  result(nil)
              }
          case "loadNativeAd": do {
                  guard let posId = dic["posId"] as? String,
                        let adWidth = dic["adWidth"] as? Double else {
                      result(nil)
                      print("空");
                      return
                  }
                  ADKleinFlutterNativeAd.shared.loadNativeAd(posId, adId: adId, width: CGFloat(adWidth))
              }
          case "loadRewardAd": do {
                      guard let posId = dic["posId"] as? String else {
                          result(nil)
                          return
                      }
                      ADKleinFlutterRewardAd.shared.load(posId: posId, adId: adId)
                      result(nil)
                  }
                  case "showRewardAd": do {
                      ADKleinFlutterRewardAd.shared.show(adId: adId)
                      result(nil)
                  }
          case "loadFullScreenVodAd": do {
                      guard let posId = dic["posId"] as? String else {
                          result(nil)
                          return
                      }
                      ADKleinFlutterFullScreenVodAd.shared.load(posId: posId, adId: adId)
                      result(nil)
                  }
                  case "showFullScreenVodAd": do {
                      ADKleinFlutterFullScreenVodAd.shared.show(adId: adId)
                      result(nil)
                  }
          case "loadIntertitialAd": do {
                      guard let posId = dic["posId"] as? String else {
                          result(nil)
                          return
                      }
                      ADKleinFlutterIntertitialAd.shared.load(posId: posId, adId: adId)
                      result(nil)
                  }
                  case "showIntertitialAd": do {
                      ADKleinFlutterIntertitialAd.shared.show(adId: adId)
                      result(nil)
                  }
          default: do {
                  result(nil)
              }
          }
      }
  }

  func getNewAdId() -> Int {
      struct Temp {
          static var adId = 0
      }
      Temp.adId -= 1
      return Temp.adId
  }

  class ADKleinFlutterAdMap {
      static let shared = ADKleinFlutterAdMap()

      var adMap = Dictionary<Int, NSObject>()
      var keyMap = Dictionary<NSObject, Int>()

      var nativeViewMap = Dictionary<Int, UIView>()
      var nativeViewKeyMap = Dictionary<UIView, Int>()

      func saveAd(ad: NSObject, adId: Int) {
          adMap[adId] = ad
          keyMap[ad] = adId
      }

      func readAd(adId: Int) -> NSObject? {
          return adMap[adId]
      }

      func readAdId(ad: NSObject) -> Int {
          return keyMap[ad] ?? -1
      }

      func deleteAd(adId: Int) {
          if let ad = readAd(adId: adId) {
              adMap.removeValue(forKey: adId)
              keyMap.removeValue(forKey: ad)
          }
      }

      // MARK: 信息流方法

      func saveNativeView(adView: UIView, adViewId: Int) {
          nativeViewMap[adViewId] = adView
          nativeViewKeyMap[adView] = adViewId
      }

      func readNativeView(adViewId: Int) -> UIView? {
          return nativeViewMap[adViewId]
      }

      func readNativeViewId(adView: UIView) -> Int {
          return nativeViewKeyMap[adView] ?? -1
      }

      func deleteNativeAdView(adViewId: Int) {
          if let adView = readNativeView(adViewId: adViewId) {
              nativeViewMap.removeValue(forKey: adViewId)
              nativeViewKeyMap.removeValue(forKey: adView)
          }
      }
  }

  public class ADKleinFlutterAdView: UIView, FlutterPlatformView {
      var adView: UIView?

      public func view() -> UIView {
          return adView ?? UIView()
      }
  }

  public class ADKleinFlutterAdViewFactory: NSObject, FlutterPlatformViewFactory {
      public func create(withFrame frame: CGRect, viewIdentifier viewId: Int64, arguments args: Any?) -> FlutterPlatformView {
          let view = ADKleinFlutterAdView()
          let adId = args as! Int
          view.adView = ADKleinFlutterAdMap.shared.readAd(adId: adId) as? UIView
          return view
      }

      public func createArgsCodec() -> FlutterMessageCodec & NSObjectProtocol {
          return FlutterStandardMessageCodec.sharedInstance()
      }
  }

  class ADKleinFlutterSplashAd: NSObject, AdKleinSDKSplashAdDelegate {
      public static let shared = ADKleinFlutterSplashAd()

      func loadAndShow(posid: String, adId: Int, imgName: String) {
          var splash = ADKleinFlutterAdMap.shared.readAd(adId: adId) as? AdKleinSDKSplashAd
          if splash == nil {
              if let window = getWindow() {
                  splash = AdKleinSDKSplashAd(placementId: posid, window: window)
                  splash!.delegate = self
                  splash!.load()
                  ADKleinFlutterAdMap.shared.saveAd(ad: splash!, adId: adId)
              }
          }
      }

      // MARK: - ADKleinSDKSplashAdDelegate

      func ak_splashAdDidFail(toPresentScreen splashAd: AdKleinSDKSplashAd, failToPresentScreen error: NSError) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: splashAd)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onFailed", arguments: ["adId": adId, "error": "错误"])
      }

      func ak_splashAdDidClose(_ splashAd: AdKleinSDKSplashAd) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: splashAd)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onClosed", arguments: ["adId": adId])
      }

      func ak_splashAdDidLoad(toPresentScreen splashAd: AdKleinSDKSplashAd) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: splashAd)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onLoad", arguments: ["adId": adId])
      }

      func ak_splashAdTimeOver(toPresentScreen splashAd: AdKleinSDKSplashAd) {
      }

      func ak_splashAdDidClick(_ splashAd: AdKleinSDKSplashAd) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: splashAd)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onClicked", arguments: ["adId": adId])
      }

      func ak_splashAdDidShow(_ splashAd: AdKleinSDKSplashAd) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: splashAd)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onExposed", arguments: ["adId": adId])
      }
  }

  class ADKleinFlutterNativeAd: NSObject, AdKleinSDKNativeExpressAdDelegate {
      static let shared = ADKleinFlutterNativeAd()

      var renderMap: NSMapTable<UIView, AdKleinSDKNativeExpressAd> = NSMapTable.strongToStrongObjects()

      func loadNativeAd(_ posId: String, adId: Int, width: CGFloat) {
          var loader = ADKleinFlutterAdMap.shared.readAd(adId: adId) as? AdKleinSDKNativeExpressAd
          if loader == nil {
              let vc = getTopViewController()
              loader = AdKleinSDKNativeExpressAd(placementId: posId, viewController: vc!)
              loader!.delegate = self
              loader!.adSize = CGSize(width: width, height: 100)
              loader!.adCount = 1
              ADKleinFlutterAdMap.shared.saveAd(ad: loader!, adId: adId)
          }
          loader!.load()
      }

      // MARK: - ADKleinSDKNativeAdDelegate
      func ak_nativeExpressAdDidLoad(_ nativeAd: AdKleinSDKNativeExpressAd,withAdViews adViewArray: [UIView & AdKleinSDKNativeExpressAdViewDelegate]) {
          print("Flutter 信息流 进行渲染广告")
          for adView in adViewArray {
              renderMap.setObject(nativeAd, forKey: adView)
              adView.render()
          }
      }

      func ak_nativeExpressAdDidRenderFail(_ adView: UIView & AdKleinSDKNativeExpressAdViewDelegate) {
          guard let loader = renderMap.object(forKey: adView) else {
              return
          }
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: loader)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onFailed", arguments: ["adId": adId])
          renderMap.removeObject(forKey: adView)
      }

      func ak_nativeExpressAdDidFail(toLoad nativeAd: AdKleinSDKNativeExpressAd, errorModel: NSError) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: nativeAd)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onFailed", arguments: ["adId": adId])
          print("Flutter 信息流 Error：\(String(errorModel.code))")
      }

      func ak_nativeExpressAdDidRenderSuccess(_ nativeAd: AdKleinSDKNativeExpressAd, adView: UIView & AdKleinSDKNativeExpressAdViewDelegate) {
          print("Flutter 信息流 加载成功")
          guard let nativeAd = renderMap.object(forKey: adView) else {
              return
          }
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: nativeAd)
          let adViewId = getNewAdId()
          ADKleinFlutterAdMap.shared.saveAd(ad: adView, adId: adViewId)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onReceived", arguments: ["adId": adId, "adViewId": adViewId, "adHeight": adView.frame.size.height, "adWidth": adView.frame.size.width])
          renderMap.removeObject(forKey: adView)
      }

      func ak_nativeExpressAdDidClose(_ nativeAd: AdKleinSDKNativeExpressAd, adView: UIView & AdKleinSDKNativeExpressAdViewDelegate) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: adView)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onClosed", arguments: ["adId": adId])
      }

      func ak_nativeExpressAdDidClick(_ nativeAd: AdKleinSDKNativeExpressAd, adView: UIView & AdKleinSDKNativeExpressAdViewDelegate) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: adView)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onClicked", arguments: ["adId": adId])
      }

      func ak_nativeExpressAdDidShow(_ nativeAd: AdKleinSDKNativeExpressAd, adView: UIView & AdKleinSDKNativeExpressAdViewDelegate) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: adView)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onExposed", arguments: ["adId": adId])
      }
  }

  class ADKleinFlutterBannerAd: NSObject, AdKleinSDKBannerAdViewDelegate{

      public static let shared = ADKleinFlutterBannerAd()
      func load(posId: String, adId: Int, adWidth:Double, adHeight:Double) {
          var bannerAd = ADKleinFlutterAdMap.shared.readAd(adId: adId) as? AdKleinSDKBannerAdView
          if bannerAd == nil {
              let vc = getTopViewController()
              bannerAd = AdKleinSDKBannerAdView(placementId: posId, viewController: vc!)
              bannerAd!.delegate = self
              bannerAd!.frame = CGRect.init(x: 0, y: 0, width: adWidth, height: adHeight)
              bannerAd!.animated = true
              ADKleinFlutterAdMap.shared.saveAd(ad: bannerAd!, adId: adId)
          }
          bannerAd?.load()

      }

      func ak_bannerAdDidLoad(_ bannerView: AdKleinSDKBannerAdView) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: bannerView)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onSucced", arguments: ["adId" : adId]);
      }

      func ak_bannerAdDidFail(toReceived bannerView: AdKleinSDKBannerAdView, errorModel: NSError) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: bannerView)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onFailed", arguments: ["adId" : adId]);
      }

      func ak_bannerAdDidClick(_ bannerView: AdKleinSDKBannerAdView) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: bannerView)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onClicked", arguments: ["adId" : adId]);
      }

      func ak_bannerAdDidClose(_ bannerView: AdKleinSDKBannerAdView) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: bannerView)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onClosed", arguments: ["adId" : adId]);
      }

      func ak_bannerAdDidShow(_ bannerView: AdKleinSDKBannerAdView) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: bannerView)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onExposed", arguments: ["adId" : adId]);
      }

  }

  class ADKleinFlutterRewardAd : NSObject, AdKleinSDKRewardVideoAdDelegate {

      public static let shared = ADKleinFlutterRewardAd()

      func load(posId: String, adId: Int) {
          var reward = ADKleinFlutterAdMap.shared.readAd(adId: adId) as? AdKleinSDKRewardVideoAd
          if reward == nil {
              reward = AdKleinSDKRewardVideoAd(placementId: posId, viewController: getTopViewController()!)
              reward?.delegate = self
              ADKleinFlutterAdMap.shared.saveAd(ad: reward!, adId: adId)
          }
          reward?.load()
      }

      func show(adId: Int) {
          if let reward = ADKleinFlutterAdMap.shared.readAd(adId: adId) as? AdKleinSDKRewardVideoAd {
              reward.show()
          }
      }

      func ak_rewardVideoAdDidSkip(_ rewardvodAd: AdKleinSDKRewardVideoAd) { }

      func ak_rewardVideoAdDidLoad(_ rewardvodAd: AdKleinSDKRewardVideoAd) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: rewardvodAd)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onSucced", arguments: ["adId" : adId]);
      }

      func ak_rewardVideoAdDidDownload(_ rewardvodAd: AdKleinSDKRewardVideoAd) { }

      func ak_rewardVideoAdDidShow(_ rewardvodAd: AdKleinSDKRewardVideoAd) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: rewardvodAd)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onExposed", arguments: ["adId" : adId]);
      }

      func ak_rewardVideoAdDidClose(_ rewardvodAd: AdKleinSDKRewardVideoAd) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: rewardvodAd)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onClosed", arguments: ["adId" : adId]);
      }

      func ak_rewardVideoAdDidClick(_ rewardvodAd: AdKleinSDKRewardVideoAd) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: rewardvodAd)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onClicked", arguments: ["adId" : adId]);
      }

      func ak_rewardVideoAdDidComplete(_ rewardvodAd: AdKleinSDKRewardVideoAd) { }

    func ak_rewardVideoAdDidFail(_ rewardvodAd: AdKleinSDKRewardVideoAd, withError: NSError) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: rewardvodAd)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onFailed", arguments: ["adId" : adId]);
      }

  }

  class ADKleinFlutterFullScreenVodAd : NSObject, AdKleinSDKFullScreenVideoAdDelegate {

      public static let shared = ADKleinFlutterFullScreenVodAd()

      func load(posId: String, adId: Int) {
          var fullScreenVodAd = ADKleinFlutterAdMap.shared.readAd(adId: adId) as? AdKleinSDKFullScreenVideoAd
          if fullScreenVodAd == nil {
              fullScreenVodAd = AdKleinSDKFullScreenVideoAd(placementId: posId, viewController: getTopViewController()!)
              fullScreenVodAd!.delegate = self
              ADKleinFlutterAdMap.shared.saveAd(ad: fullScreenVodAd!, adId: adId)
          }
          fullScreenVodAd!.load()
      }

      func show(adId: Int) {
          if let fullScreenVodAd = ADKleinFlutterAdMap.shared.readAd(adId: adId) as? AdKleinSDKFullScreenVideoAd {
              fullScreenVodAd.show()
          }
      }

      func ak_fullScreenVideoAdDidLoad(_ fullScreenVodAd: AdKleinSDKFullScreenVideoAd) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: fullScreenVodAd)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onSucced", arguments: ["adId" : adId]);
      }

      func ak_fullScreenVideoAdDidSkip(_ fullScreenVodAd: AdKleinSDKFullScreenVideoAd) {
      }

      func adsy_fullScreenVodAdDidPresent(_ fullScreenVodAd: AdKleinSDKFullScreenVideoAd) {
      }

      func ak_fullScreenVideoAdDidFail(toLoad fullScreenVodAd: AdKleinSDKFullScreenVideoAd, errorModel: NSError) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: fullScreenVodAd)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onFailed", arguments: ["adId" : adId]);
      }

      func ak_fullScreenVideoAdDidClick(_ fullScreenVodAd: AdKleinSDKFullScreenVideoAd) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: fullScreenVodAd)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onClicked", arguments: ["adId" : adId]);
      }

      func ak_fullScreenVideoAdDidClose(_ fullScreenVodAd: AdKleinSDKFullScreenVideoAd) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: fullScreenVodAd)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onClosed", arguments: ["adId" : adId]);
      }

      func ak_fullScreenVideoAdDidShow(_ fullScreenVodAd: AdKleinSDKFullScreenVideoAd) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: fullScreenVodAd)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onExposed", arguments: ["adId" : adId]);
      }

      func ak_fullScreenVideoAdDidComplete(_ fullScreenVodAd: AdKleinSDKFullScreenVideoAd) {
      }

  }

  class ADKleinFlutterIntertitialAd : NSObject, AdKleinSDKInterstitialAdDelegate {

      public static let shared = ADKleinFlutterIntertitialAd()

      func load(posId: String, adId: Int) {
          var inter = ADKleinFlutterAdMap.shared.readAd(adId: adId) as? AdKleinSDKInterstitialAd
          if inter == nil {
              inter = AdKleinSDKInterstitialAd(placementId: posId, viewController: getTopViewController()!)
              inter?.delegate = self
              ADKleinFlutterAdMap.shared.saveAd(ad: inter!, adId: adId)
          }
          inter?.load()
      }

      func show(adId: Int) {
          if let inter = ADKleinFlutterAdMap.shared.readAd(adId: adId) as? AdKleinSDKInterstitialAd {
              inter.show()
          }
      }

      func ak_interstitialAdDidLoad(_ interstitialAd: AdKleinSDKInterstitialAd) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: interstitialAd)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onSucced", arguments: ["adId" : adId]);
      }

      func ak_interstitialAdDidFail(toLoad interstitialAd: AdKleinSDKInterstitialAd, error: NSError) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: interstitialAd)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onFailed", arguments: ["adId" : adId]);
      }

      func ak_interstitialAdDidRenderSuccess(_ interstitialAd: AdKleinSDKInterstitialAd) { }

      func ak_interstitialAdDidRenderFail(toPresent interstitialAd: AdKleinSDKInterstitialAd, error: NSError) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: interstitialAd)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onFailed", arguments: ["adId" : adId]);
      }

      func ak_interstitialAdDidClick(_ interstitialAd: AdKleinSDKInterstitialAd) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: interstitialAd)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onClicked", arguments: ["adId" : adId]);
      }

      func ak_interstitialAdDidClose(_ interstitialAd: AdKleinSDKInterstitialAd) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: interstitialAd)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onClosed", arguments: ["adId" : adId]);
      }

      func ak_interstitialAdDidShow(_ interstitialAd: AdKleinSDKInterstitialAd) {
          let adId = ADKleinFlutterAdMap.shared.readAdId(ad: interstitialAd)
          SwiftAdKleinFlutterSdkPlugin.channel!.invokeMethod("onExposed", arguments: ["adId" : adId]);
      }
  }


  // MARK: Helper

  func getWindow() -> UIWindow? {
      return UIApplication.shared.keyWindow
  }

  func getTopViewController() -> UIViewController? {
      if let window = UIApplication.shared.keyWindow {
          return getTopViewController(from: window)
      }
      return nil
  }

  func getTopViewController(from window: UIWindow) -> UIViewController? {
      if let rootVc = window.rootViewController {
          return getTopViewController(from: rootVc)
      }
      return nil
  }

  func getTopViewController(from vc: UIViewController) -> UIViewController {
      if let newVc = vc.presentedViewController {
          return getTopViewController(from: newVc)
      }
      if let tvc = vc as? UITabBarController {
          if let newVc = tvc.selectedViewController {
              return getTopViewController(from: newVc)
          }
      }
      if let nvc = vc as? UINavigationController {
          if let newVc = nvc.topViewController {
              return getTopViewController(from: newVc)
          }
      }
      return vc
  }
