import 'dart:collection';
import 'dart:io';

import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';

import 'ad_klein_ad.dart';
import 'ad_klein_flutter_sdk.dart';

abstract class ADKleinAd {
  int adId = 0;

  void Function()? onSucced;
  void Function()? onFailed;
  void Function()? onClosed;
  void Function()? onClicked;
  void Function()? onExposed;

  // rewardAd only
  void Function()? onRewarded;

  // nativeAd only
  void Function(ADKleinFlutterNativeAdView adView)? onReceived;

  void regist() {
    adId = instanceManager.regist(this);
  }

  void unregist() {
    AdKleinFlutterSdk.releaseAd(adId: this.adId);
    instanceManager.unregist(this);
  }
}

class ADKleinAdView extends ADKleinAd {
  double width = 1080;
  double height = 0;
}

class ADKleinAdLoader extends ADKleinAd {}

ADKleinAdManager instanceManager = ADKleinAdManager();

class ADKleinAdManager {
  HashMap<int, dynamic> map = new HashMap();
  int adId = 0;

  int regist(ADKleinAd ad) {
    int adId = ad.adId;
    if (ad.adId == 0) {
      this.adId += 1;
      adId = this.adId;
    }
    map[adId] = ad;
    return adId;
  }

  void unregist(ADKleinAd ad) {
    adId = ad.adId;
    map.remove(adId);
  }

  int adIdFor(ADKleinAd ad) {
    return ad.adId;
  }

  ADKleinAd adFor(int adId) {
    return map[adId];
  }
}

class ADKleinWidget extends StatelessWidget {
  final ADKleinAdView adView;

  ADKleinWidget({Key? key, required this.adView}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    if (Platform.isIOS) {
      return Container(
          width: this.adView.width,
          height: this.adView.height,
          child: UiKitView(
              viewType: "ADKleinAdView",
              creationParams: instanceManager.adIdFor(this.adView),
              creationParamsCodec: StandardMessageCodec()));
    } else if (Platform.isAndroid) {
      return Container(
          width: this.adView.width,
          height: this.adView.height,
          child: AndroidView(
              viewType: "ADKleinAdView",
              creationParams: instanceManager.adIdFor(this.adView),
              creationParamsCodec: StandardMessageCodec()));
    }
    return Container();
  }
}
